package com.training.aop.aspects;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.springframework.stereotype.Component;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Aspect
@Component
public class LoggerAspect {
	private static final Logger log = LoggerFactory.getLogger(LoggerAspect.class);

	@Around("execution(* com.training.dao.*.*(..))")
	public Object logExecutionTime(ProceedingJoinPoint joinPoint) throws Throwable {
		long startTime = System.currentTimeMillis();

		Object proceed = joinPoint.proceed();

		long endTime = System.currentTimeMillis();
		log.debug(joinPoint.getSignature() + " executed in " + (endTime - startTime) + "ms");

		return proceed;
	}

	@Before("execution(* com.training.dao.*.*(..))")
	public void logBeforeMethods(JoinPoint joinPoint) {
		log.info("Advice Before Class : " + joinPoint.getSignature() + " : Logging before method");
	}

	@After("execution(* com.training.dao.*.*(..))")
	public void logAfterMethods(JoinPoint joinPoint) {
		log.info("Advice After Class : " + joinPoint.getSignature() + " : Logging before method");
	}

	@AfterReturning(pointcut = "execution(* com.training.dao.*.*(..))", returning = "result")
	public void afterReturningAdvice(JoinPoint joinPoint, Object result) {
		log.info("Advice AfterReturning Class : " + joinPoint.getSignature() + " , result : " + result);
	}

	@AfterThrowing(pointcut = "execution(* com.training.dao.*.*(..))", throwing = "ex")
	public void afterThrowingAdvice(JoinPoint joinPoint, Exception ex) {
		log.error("Advice AfterThrowing  Class : " + joinPoint.getSignature() + " , Exception Catched in Aspect : "
				+ ex.getMessage());
	}
}