package com.training.dao;

import com.training.entities.Personne;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.stream.Collectors;

@Repository
public class PersonneDao implements IPersonneDao {

	@Autowired
	private List<Personne> personnesListDataSource;

	@Override
	public Personne create(Personne person) {
		if (person != null) {
			person.setIdPersonne(getMaxId() + 1);
			if (!personnesListDataSource.add(person)) {
				person = null;
			}
		}
		return person;
	}

	public Integer getMaxId() {
		Integer maxId = Integer.MIN_VALUE;
		for (int i = 0; i < personnesListDataSource.size(); i++) {
			Personne person = personnesListDataSource.get(i);
			if (person.getIdPersonne() > maxId) {
				maxId = person.getIdPersonne();
			}
		}
		return maxId;
	}

	@Override
	public Personne update(Personne person) {
		Personne personToReturn = null;
		if (person != null) {
			for (int i = 0; i < personnesListDataSource.size(); i++) {
				Personne userFromList = personnesListDataSource.get(i);
				if (person.getIdPersonne().equals(userFromList.getIdPersonne())) {
					personnesListDataSource.set(i, person);
					personToReturn = person;
				}
			}
		}
		return personToReturn;
	}

	@Override
	public boolean delete(Personne person) {
		boolean isOk = false;
		if (person != null) {
			Personne tempPersonnes = findById(person.getIdPersonne());
			if (tempPersonnes != null) {
				isOk = personnesListDataSource.remove(tempPersonnes);
			}
		}
		return isOk;
	}

	@Override
	public List<Personne> findAll() {
		return personnesListDataSource;
	}

	@Override
	public Personne findById(Integer id) {
		Personne personReturn = null;
		List<Personne> myPersons = personnesListDataSource.stream().filter(person -> person.getIdPersonne().equals(id))
				.collect(Collectors.toList());
		if (myPersons != null && !myPersons.isEmpty()) {
			personReturn = myPersons.get(0);
		}
		return personReturn;
	}

	@Override
	public List<Personne> findByPrenom(String prenom) {
		return personnesListDataSource.stream().filter(person -> person.getPrenom().equalsIgnoreCase(prenom))
				.collect(Collectors.toList());
	}

	@Override
	public List<Personne> findByNom(String nom) {
		return personnesListDataSource.stream().filter(person -> person.getNom().equalsIgnoreCase(nom))
				.collect(Collectors.toList());
	}

	@Override
	public List<Personne> findByPrenomNom(String prenom, String nom) {
		List<Personne> myPersons = personnesListDataSource.stream()
				.filter(person -> person.getPrenom().equalsIgnoreCase(prenom))
				.filter(person -> person.getNom().equalsIgnoreCase(nom)).collect(Collectors.toList());
		return myPersons;
	}

	@Override
	public List<Personne> findByCodePostal(String codePostal) {
		return personnesListDataSource.stream().filter(person -> person.getCodePostal().equalsIgnoreCase(codePostal))
				.collect(Collectors.toList());
	}

	@Override
	public List<Personne> findByVille(String ville) {
		return personnesListDataSource.stream().filter(person -> person.getVille().equalsIgnoreCase(ville))
				.collect(Collectors.toList());
	}

	@Override
	public List<Personne> findPersonnesObese() {
		return personnesListDataSource.stream().filter(Personne::isObese).collect(Collectors.toList());
	}

	@Override
	public List<Personne> findPersonnesMaigre() {
		return personnesListDataSource.stream().filter(Personne::isMaigre).collect(Collectors.toList());
	}

	@Override
	public void simulateThrowException() {
		throw new NullPointerException("Throwing NullPointerException");
	}
}