package com.training.entities;

import com.training.utils.Constants;

public class Personne implements Comparable<Personne> {
	private Integer idPersonne;
	private String prenom;
	private String nom;
	private Double poids;
	private Double taille;
	private String rue;
	private String ville;
	private String codePostal;
	private String pays;

	public Personne() {

	}

	public Personne(String prenom, String nom, Double poids, Double taille, String rue, String ville, String codePostal,
			String pays) {
		this.prenom = prenom;
		this.nom = nom;
		this.poids = poids;
		this.taille = taille;
		this.rue = rue;
		this.ville = ville;
		this.codePostal = codePostal;
		this.pays = pays;
	}

	public Personne(Integer idPersonne, String prenom, String nom, Double poids, Double taille, String rue,
			String ville, String codePostal, String pays) {
		this.idPersonne = idPersonne;
		this.prenom = prenom;
		this.nom = nom;
		this.poids = poids;
		this.taille = taille;
		this.rue = rue;
		this.ville = ville;
		this.codePostal = codePostal;
		this.pays = pays;
	}

	public Integer getIdPersonne() {
		return idPersonne;
	}

	public void setIdPersonne(Integer idPersonne) {
		this.idPersonne = idPersonne;
	}

	public String getNom() {
		return nom;
	}

	public void setNom(String nom) {
		this.nom = nom;
	}

	public Double getPoids() {
		return poids;
	}

	public void setPoids(Double poids) {
		this.poids = poids;
	}

	public Double getTaille() {
		return taille;
	}

	public void setTaille(Double taille) {
		this.taille = taille;
	}

	public String getPrenom() {
		return prenom;
	}

	public void setPrenom(String prenom) {
		this.prenom = prenom;
	}

	public String getRue() {
		return rue;
	}

	public void setRue(String rue) {
		this.rue = rue;
	}

	public String getVille() {
		return ville;
	}

	public void setVille(String ville) {
		this.ville = ville;
	}

	public String getCodePostal() {
		return codePostal;
	}

	public void setCodePostal(String codePostal) {
		this.codePostal = codePostal;
	}

	public String getPays() {
		return pays;
	}

	public void setPays(String pays) {
		this.pays = pays;
	}

	@Override
	public String toString() {
		return String.format(
				"[idPersonne=%s, prenom=%s, nom=%s, poids=%s, taille=%s, rue=%s, ville=%s, codePostal=%s, pays=%s]",
				idPersonne, prenom, nom, poids, taille, rue, ville, codePostal, pays);
	}

	@Override
	public int hashCode() {
		// int hash = prenom.hashCode() + nom.hashCode();
		// hash = (prenom+nom).hashCode();
		// Objects.hash(prenom, nom);
		// this.getIdPersonne();
		int prime = 31;
		int result = 1;
		result = result * prime + this.prenom.hashCode();
		result = result * prime + this.nom.hashCode();
		return result;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null) {
			return false;
		}
		if (o instanceof Personne) {
			// Objects.equals(((Personne) o).getPrenom() + ((Personne) o).getNom(),
			// getPrenom() + getNom());
			return this.prenom.equals(((Personne) o).prenom) && this.nom.equals(((Personne) o).nom);
		}
		return false;
	}

	@Override
	public int compareTo(Personne otherPersonne) {
		return this.getIdPersonne().compareTo(otherPersonne.getIdPersonne());
	}

	public String toFormatText() {
		return this.getIdPersonne() + Constants.CSV_SEPARATOR + " " + this.getPrenom() + Constants.CSV_SEPARATOR + " "
				+ this.getNom() + Constants.CSV_SEPARATOR + " " + this.getPoids() + Constants.CSV_SEPARATOR + " "
				+ this.getTaille() + Constants.CSV_SEPARATOR + " " + this.getRue() + Constants.CSV_SEPARATOR + " "
				+ this.getVille() + Constants.CSV_SEPARATOR + " " + this.getCodePostal();
	}

	public double getImc() {
		double imc = 0;
		if (taille != 0) {
			imc = poids / Math.pow(taille / 100, 2);
		}
		return imc;
	}

	public boolean isMaigre() {
		double imc = getImc();
		return imc > Constants.LIMITE_INF_MAIGRE && imc < Constants.LIMITE_SUP_MAIGRE;
	}

	public boolean isSurPoids() {
		double imc = getImc();
		return imc > Constants.LIMITE_INF_SURPOIDS && imc < Constants.LIMITE_SUP_SURPOIDS;
	}

	public boolean isObese() {
		double imc = getImc();
		return imc > Constants.LIMITE_SUP_SURPOIDS;
	}
}